
/*
 *  Copyright  (c)  L.Allison, C.S.Wallace, C.N.Yee.
 *     Dept. Computer Science, Monash University, Australia 3168,    May 1991.
 *  1. These routines may only be used for non-commercial, non-classified,
 *     research purposes.
 *  2. No liability of any kind is accepted for their use.
 *  3. As a courtesy, please inform us of any interesting uses of the routines,
 *     of any bugs that you find in them and of how to correct
 *     such bugs, if known.
 *  4. Do not delete this copyright notice.
 *  5. Please include the following references in any publication based wholly
 *     or in part on the use of these routines or on the ideas contained in them:
 *   . Allison L. & C.N.Yee.
 *        Minimum Message Length Encoding and the Comparison of Macro-Molecules.
 *        Bull. Math. Bio. 52(3) 431-453 1990
 *   . Allison L., C.S.Wallace & C.N.Yee. Finite-State Models in the Alignment
 *        of Macromolecules. J. Mol. Evol. (1992) 35:77-89
 *  6. You may distribute the routines to others if and only if
 *     they agree to accept and maintain these conditions and no fee is charged.
 *  7. There is no rule 7.
 */
#define IDLength	40
#define DESCLength	180

#define MAXSEQLENGTH	4000
#define MAXALIGNLENGTH	8000	/* 2 * MAXSEQLENGTH */
#define MATRIXSIZE		650

#define GENBANKDELIMITER '>' 	/* Genbank record delimiter */

#define MaxSymbol	257

#define	NULLSymbol	0

typedef int	symbol ;

typedef struct {
	int	size ;		/* need this for binary search */
	char	Symbol[MaxSymbol] ;/* must be initialized in ascending order*/
	} AlphabetTable ; 

typedef struct {
	int	size ;
	int	complement[MaxSymbol] ;
	} ComplementTable ;


typedef	struct {
	char identifier[IDLength] ;
	char description[DESCLength] ;
	int length ;
	symbol *sequence ;	/* malloc the spaces required */
	} string ;

typedef struct {	/* alignment of 2 strings */
	int a, b ;
	} align ;

typedef struct {	/* alignment of 3 strings */
	int a, b, c ;
	} align3D ;

typedef struct {	/* for block alignment */
	int d, v, h ;
	} Balign ;

extern AlphabetTable 	DNASymbol, AlphaSymbol ;
extern ComplementTable	DNAComplement ;

extern AlphabetTable *SymbTable ;

float Cost11(), Cost21() ;	/* Cost functions */

extern double logfactorial[MAXSEQLENGTH] ;
