
/*
 *  Copyright  (c)  L.Allison, C.S.Wallace, C.N.Yee.
 *     Dept. Computer Science, Monash University, Australia 3168,    May 1991.
 *  1. These routines may only be used for non-commercial, non-classified,
 *     research purposes.
 *  2. No liability of any kind is accepted for their use.
 *  3. As a courtesy, please inform us of any interesting uses of the routines,
 *     of any bugs that you find in them and of how to correct
 *     such bugs, if known.
 *  4. Do not delete this copyright notice.
 *  5. Please include the following references in any publication based wholly
 *     or in part on the use of these routines or on the ideas contained in them:
 *   . Allison L. & C.N.Yee.
 *        Minimum Message Length Encoding and the Comparison of Macro-Molecules.
 *        Bull. Math. Bio. 52(3) 431-453 1990
 *   . Allison L., C.S.Wallace & C.N.Yee. Finite-State Models in the Alignment
 *        of Macromolecules. J. Mol. Evol. (1992) 35:77-89
 *  6. You may distribute the routines to others if and only if
 *     they agree to accept and maintain these conditions and no fee is charged.
 *  7. There is no rule 7.
 */
/*
 *	"misc.c"  miscellaneous routines
 *
 *	SellersProb(PMatches,PChanges,PInsDel)
 *		float	*PMatches,*PChanges,*PInsDel ;
 *		returns a set of probability that is equivalent to Seller's metric 
 *
 *	changetoprobability3(n1,n2,n3,p1,p2,p3)
 *		int	n1,n2,n3 ; 	float	*p1,*p2,*p3 ;
 *		change a set of counts to a set of probability
 *
 *	changetoprobability4(n1,n2,n3,n4,p1,p2,p3,p4)
 *		int	n1,n2,n3,n4 ; 	float	*p1,*p2,*p3,*p4 ;
 *		change a set of counts to a set of probability
 *
 *	long lmin3(x, y, z)
 *		long x, y, z ;
 *
 *	int min3(x, y, z)
 *		int x, y, z ;
 *
 *	float fmin3(x, y, z), flabs(x, y)
 *		float x, y, z ;
 *
 *	double log2(x)
 *		double x ;
 *
 *	double logstar(x)
 *		int x ;
 *
 *	double riss(x)
 *		int x ;
 *
 *	double yee(x)
 *		int x ;
 *
 *	double lloyd(x)
 *		int x ;
 *		Hybrid of smoothen logstar and yee
 *
 *	double geometric(P, x)
 *		float	P ;
 *		int x ;
 *
 *	int randomnum(n)
 *	int n ;
 *		this routine returns a random number of range 0..n-1
 *		randomseed must be called before calling this routine
 *
 *	randomseed(seed)
 *	int seed ; 
 *		provide the random number with a starting seed
 */	
#include <math.h>
#include "Strings.h"
#include "macro.h"

/*
double	ln2, LogRissConstant, RissConstant, YeeConstant, LogYeeConstant ;
*/
long time() ;
double	ln2 = 0.69314718055994530941 ;
double	logDelta ;
static int tablesize = 0 ;
/*
static double	risstable[MAXSEQLENGTH] ;
static double	yeetable[MAXSEQLENGTH] ;
*/
static double	lloydtable[MAXSEQLENGTH] ;

long lmin3(x,y,z)
	long x,y,z ;
{
	long t ;
	t = (x < y) ? x : y ;
	return (t < z)? t : z ;
}

int min7(a,b,c,d,e,f,g)
	int a,b,c,d,e,f,g ;
{
	int	t,u,v,x,y ;

	t = min(a,b) ; u = min(c,d) ; v = min(e,f) ;

	x = min(t,u) ; y = min(v,g) ;
	return min(x,y) ;
}

int min3(x, y, z)
	int	x, y, z ;
{
	int	t ;

	t = (x < y) ? x : y ;
	return (t < z)? t : z ;
}

float flabs(a)
	float a ;
{
	if (a < 0) return -a ;
	else return a ;
}

float fmin2(x, y)
	float	x, y ;
{
	return x < y ? x : y;
}

float fmin3(x, y, z)
	float	x, y, z ;
{
	float	t ;

	t = (x < y) ? x : y ;
	return (t < z)? t : z ;
}

float fmin5(a, b, c, d, e)
	float	a, b, c, d, e ;
{ return fmin3(min(a,b),min(c,d),e) ; }

float fmax3(x, y, z)
	float	x, y, z ;
{
	float	t ;

	t = (x > y) ? x : y ;
	return (t > z)? t : z ;
}

/*
 *	generates a random number from 0..n-1
 */
int randomnum(n)
	int n ;
{
	int t ;

	t = rand()/1000 ; /* get rid of bottom 3 digits */
	return t % n ;
}

randomseed(seed)
	int	seed ;
{ srand(seed) ; }

/*
 *	use system clock to initialize the random number generator
 */
generaterandomseed(seed)
	int *seed ;
{
	long clock ;
	clock = time((long *) 0) ;
	*seed = (int) clock ;
	randomseed(*seed) ;
}

double	exp2(x)
	double	x ;
{ return exp(x*ln2) ; }

double log2(x)
	double x ;
{ return (double) (log(x) / ln2) ; }

double geometric(P, x) 
	float	P ;
	int	x ;
{
	return (x-1)*log2((double)P) + log2((double)1-P) ;
}

double logstar(x)
	int	x ;
{
	double	y, u ;

	u = (double) x ;
	y = 0 ;
	while (u > 1.0) {
		u = log2(u) ;
		y += u ;
	}
	return y ;
}

/*
static double ris(x)
	int	x ;
{ 
	return logstar(x) + LogRissConstant ;
}

double riss(x)
	int	x ;
{
	if ( x < tablesize) return risstable[x] ;
	else return ris(x) ;
}
*/


/* log(x) + log^2(x+1) + log^3(x+3) + log^4(x+15) + ... */
static double yeelog(x) 
	int x ; /* x>=1 */
{
	int 	i, j ;
	double	z, y ;
	static double	power[] = {1, 2, 4, 16, 65536 /*64*1024*/} ;

	y = 0;
	for (i = 0 ; i < 5; i++) {
		z = (double) (x + power[i] - 1) ;
	    for (j = 0 ; j <= i; j++) z = log2(z);
	    y += z ;
	}
	return y ;
}

/*
double yees(x)
	int x ;
{
	return yeelog(x) + LogYeeConstant ;
}

double yee(x)
	int	x ;
{
	if ( x < tablesize) return yeetable[x] ;
	else return yees(x) ;
}
*/

double lloyds(x)
	int	x ;
{
   double	ans ;

	if (x <= 8)  {
		if (x == 2 || x == 4)
			ans = 0.45*logstar(x-1)+0.55*logstar(x+1) ; /* concave */
	 	else ans = logstar(x);
	 	ans += (yeelog(9)-logstar(9)) ; /* smooth join */
	} else /* x>8 */
         ans = yeelog(x);

    return ans + logDelta ;
}

double lloyd(x)
	int	x ;
{
	if ( x < tablesize) return lloydtable[x] ;
	else return lloyds(x) ;
}

loginit(size)
	int	size ;
{
	int	i ;

	if (size > MAXSEQLENGTH) {
		printf("loginit: too long\n") ;
		exit(99) ;
	}

/*
	RissConstant = 2.865064 ;
	LogRissConstant = log2(RissConstant) ;
	YeeConstant = 2.37564 ;
	LogYeeConstant = log2(YeeConstant) ;
*/

   	logDelta = log2((double)2.279);
	tablesize = size ;

/*
	risstable[0] = 0 ;
	for (i=1 ; i <= size; i++)
		risstable[i] = ris(i) ;

	yeetable[0] = 0 ;
	for (i=1 ; i <= size; i++)
		yeetable[i] = yees(i) ;
*/

	lloydtable[0] = 0 ;
	for (i=1 ; i <= size; i++)
		lloydtable[i] = lloyds(i) ;
}

/*--------------------------------------------------------------------------*/

/*
 *	this routine return a set of probability that for R-model
 *	will give the same alignment as Seller's metric
 */
SellersProb(PMatches,PChanges,PInsDel)
	float *PMatches,*PChanges,*PInsDel ;
{
	*PMatches = 0.5743492 ;
	*PChanges = 0.1542580 ;
	*PInsDel = 1 - *PMatches - *PChanges ;
}

#define SENSITIVITY 0.5

changetoprobability3(n1,n2,n3,p1,p2,p3)
	int	n1,n2,n3 ;
	float	*p1,*p2,*p3 ;
{
	float sum ;
	sum = n1 + n2 + n3 + 3*SENSITIVITY ;
	*p1 = (n1+SENSITIVITY)/sum ;
	*p2 = (n2+SENSITIVITY)/sum ;
	*p3 = (n3+SENSITIVITY)/sum ;
}

fchangetoprobability2(n1,n2,p1,p2)
	float	n1,n2,*p1,*p2 ;
{ 
	*p1 = (n1+SENSITIVITY)/(n1+n2+2*SENSITIVITY) ;
	*p2 = (n2+SENSITIVITY)/(n1+n2+2*SENSITIVITY) ;
}

fchangetoprobability3(n1,n2,n3,p1,p2,p3)
	float	n1,n2,n3,*p1,*p2,*p3 ;
{
	float sum ;
	sum = n1 + n2 + n3 + 3*SENSITIVITY ;
	*p1 = (n1+SENSITIVITY)/sum ;
	*p2 = (n2+SENSITIVITY)/sum ;
	*p3 = (n3+SENSITIVITY)/sum ;
}

fchangetoprobability4(n1,n2,n3,n4,p1,p2,p3,p4)
	float	n1,n2,n3,n4 ;
	float	*p1,*p2,*p3,*p4 ;
{
	float sum ;
	sum = n1+n2+n3+n4+4*SENSITIVITY ;
	*p1 = (n1+SENSITIVITY)/sum ; *p2 = (n2+SENSITIVITY)/sum ;
	*p3 = (n3+SENSITIVITY)/sum ; *p4 = (n4+SENSITIVITY)/sum ;
}

fchangetoprobability6(n1,n2,n3,n4,n5,n6,p1,p2,p3,p4,p5,p6)
	float	n1,n2,n3,n4,n5,n6 ;
	float	*p1,*p2,*p3,*p4,*p5,*p6 ;
{
	float sum ;
	sum = n1+n2+n3+n4+n5+n6+6*SENSITIVITY ;
	*p1 = (n1+SENSITIVITY)/sum ; *p2 = (n2+SENSITIVITY)/sum ;
	*p3 = (n3+SENSITIVITY)/sum ; *p4 = (n4+SENSITIVITY)/sum ;
	*p5 = (n5+SENSITIVITY)/sum ; *p6 = (n6+SENSITIVITY)/sum ;
}


/*---------------------------------------------------------------------*/
float lgstar(x)
	float x ;
{
	if (x > 1) {
		x = (float)log2((double)x) ;
		return x + lgstar(x) ;
	} else return 0 ;
}

/*
 *	increase in mes' len' due to using an est' of prob's rather than true
 *	prob's which are of course unknown
 */
static float errorcost(K)
	int K ;
{ return (float)((K-1.0)/2.0 * log2(exp((double)1))) ;}


/*
 *	estimate cost of transmitting probability of params in a header.
 *	The parameter has 6 values, 5 degrees of freedom.
 */
float paramcost6(fm/*atch*/, fc/*hange*/, fh/*horiz*/, fv/*ert*/,fa,fb)
	float	fm, fc, fh, fv, fa, fb ;
{
	float V, deltaVsq, fsum, pm, pc, ph, pv, pa, pb ;
	int K ;

	K = 6 ;
	fsum = fm+fc+fh+fv+fa+fb ;
	fsum += 0.05 ; /* in case fsum is 0 */
	fchangetoprobability6(fm,fc,fh,fv,fa,fb,&pm,&pc,&ph,&pv,&pa,&pb) ;
	deltaVsq =  (pm*pc*ph*pv*pa*pb) / exp((double)(K-1)*log((double)fsum/12));
	V = 1.0/(5*4*3*2*1) ; /*=1/(K-1)!*/

	return  (float)(log((double)1 + V*V/deltaVsq)/2.0/log((double)2) + errorcost(K)) ;
}

/*
 *	estimate cost of transmitting probability of params in a header.
 *	The parameter has 4 values, 3 degrees of freedom.
 */
float paramcost4(/*frequencies*/ fm/*atch*/, fc/*hange*/, fh/*horiz*/, fv/*ert*/)
	float	fm, fc, fh, fv ;
{
	float V, deltaVsq, fsum, pm, pc, ph, pv ;
	int	K ;

	K = 4 ;
	fsum = fm+fc+fh+fv ;
	fsum += 0.05 ; /* in case fsum is 0 */
	fchangetoprobability4(fm,fc,fh,fv,&pm,&pc,&ph,&pv) ;
	deltaVsq =  (pm*pc*ph*pv) / exp((double)(K-1)*log((double)fsum/12));
	V = 1.0/(3.0*2.0) ; /*=1/(4-1)!*/
	return  (float)(log((double)1 + V*V/deltaVsq)/2.0/log((double)2)+errorcost(K)) ;
}

float paramcost2(fm, fc)
	float fm, fc ;
{
	float V, deltaVsq, fsum, pm, pc ;
	int K ;

	K = 2 ;
	fsum = fm+fc;
	fsum += 0.05 ; /* in case fsum == 0 */
	fchangetoprobability2(fm,fc,&pm,&pc) ;
	deltaVsq = (float)(pm*pc/exp((double)(K-1)*log((double)fsum/12)));
	V = 1.0/1.0 ; /* = 1/(K-1)! */
	return (float)(log((double)1 + V*V/deltaVsq)/2.0/log((double)2) + errorcost(K)) ;
}


float paramcost3(fm, fc, fid)
	float fm, fc, fid ;
{
	float V, deltaVsq, fsum, pm, pc, pid ;
	int K ;

	K = 3 ;
	fsum = fm+fc+fid;
	fsum += 0.05 ; /* in case fsum == 0 */
	fchangetoprobability3(fm,fc,fid,&pm,&pc,&pid) ;
	deltaVsq = (float)(pm*pc*pid/exp((double)(K-1)*log((double)fsum/12)));
	V = 1.0/2.0 ; /* = 1/(K-1)! */
	return (float)(log((double)1 + V*V/deltaVsq)/2.0/log((double)2)+errorcost(K)) ;
}

/* 
 *	Code for integers ~ universal prior
 */
float U( L )
	float L ; /* L>=1 */
{ return (float) (lgstar(L) + log2((double)2.865)) ;}


/*
 *	Binomial Code for len1 - len2
 *
 *       1          1                 n=0
 *     1   1        2                 n=1
 *   1   2   1      4                 n=2
 * 1   3   3   1  = 8 = 2**3          n=3
 */
float DiffCode(len1, len2 )
	int	len1, len2 ;
{
	int i, n, r, sum, diff ;
	float	codeword ;

	sum = len1+len2; diff = len1-len2 /*can be + or -*/;
	/*NB. sum is even <=> diff is even*/
	n = sum; r = (sum+diff) / 2 /* >= 0 and <= n */;
	codeword = n;  /*p(r) = C(n,r)/2**n*/
	for (i = r+1; i <= n; i++ ) codeword = codeword - log2((double)i);
	for (i = 1 ; i <= n-r; i++) codeword = codeword + log2((double)i);
	return codeword ;
}

/*
 *	Compute the null theory message length for two strings of length m and n
 */
float NullMesgLength(m,n)
{
	return (float)U((float)m+n)+DiffCode(m,n)+(m+n)*2;
}
