
/*
 *  Copyright  (c)  L.Allison, C.S.Wallace, C.N.Yee.
 *     Dept. Computer Science, Monash University, Australia 3168,    May 1991.
 *  1. These routines may only be used for non-commercial, non-classified,
 *     research purposes.
 *  2. No liability of any kind is accepted for their use.
 *  3. As a courtesy, please inform us of any interesting uses of the routines,
 *     of any bugs that you find in them and of how to correct
 *     such bugs, if known.
 *  4. Do not delete this copyright notice.
 *  5. Please include the following references in any publication based wholly
 *     or in part on the use of these routines or on the ideas contained in them:
 *   . Allison L. & C.N.Yee.
 *        Minimum Message Length Encoding and the Comparison of Macro-Molecules.
 *        Bull. Math. Bio. 52(3) 431-453 1990
 *   . Allison L., C.S.Wallace & C.N.Yee. Finite-State Models in the Alignment
 *        of Macromolecules. J. Mol. Evol. (1992) 35:77-89
 *  6. You may distribute the routines to others if and only if
 *     they agree to accept and maintain these conditions and no fee is charged.
 *  7. There is no rule 7.
 */
/*
 *	"Printimage.c"
 *	transform a greyscale image into poscript for lazer writer
 *
 *	input :
 *		height width   of image 
 *		Astring
 *		Bstring
 *		[height-1,0], ... , [height-1,width-1]
 *				.
 *				.
 *				.
 *		[0,0],        ... , [0, width-1]
 *		Caption - optional,  if present will be printed at bottom of page
 */
#include <stdio.h>
#include "macro.h"

#define ImageSize 450	/* ~6.5 inches,  72 pixels/inch */
#define LegendSpace 216 /* 3 inches,     ""          */
#define Xmargin 80
#define Ymargin 80
#define LegendDot 25

char *FontType = "Courier-Bold" ;

#define MAXSEQLENGTH 2000

int width, height ;
int usersize ;
float PixelsPerDot ;
int x, y, Ymax, fontsize ;
char A[MAXSEQLENGTH], B[MAXSEQLENGTH] ;
char Label[100] ;

int i, j ;

readln()
{
	while (getchar() != '\n') ;
}

readstring(str)
	char *str ;
{
	int i ;

	while ((*str = getchar()) != '\n' && *str != EOF) str++ ;
	*str = '\0' ;
}


/*
 *	determines the pointsize to use for labelling the density plot 
 */
int FontSize(size)
	float size ;
{ return (int) size+1 ; }

printhexdigit(n)
	int n ;
{
	if (n <= 9) putchar((char)(n+'0')) ;
	else putchar((char)(n-10+'a')) ;
}

/*
 *	change a decimal number to hex ... works only for two digits number!!
 */
printhex(value)
	int value ;
{
	printhexdigit(value/16) ;
	printhexdigit(value%16) ;
}

setfont(fonttype, fontsize)
	char *fonttype ;
	int fontsize ;
{
	printf("/%s findfont\n",fonttype) ;
	printf("%d scalefont setfont\n", fontsize) ;
}

/*
 *	left justify a string at x,y
 */
LeftJustify(s, x, y) 
	char *s ;
	int x, y ;
{
	printf("%d ",x) ;
	printf("(%s) stringwidth\n",s);
	printf("pop sub\n") ;
	printf("%d moveto\n",y) ; /* y coordinate */
	printf("(%s) show\n", s);
}

/*
 *	generates postscript command to center a string at x, y in a space of width
 */
centerstring(s, x, y, width)
	char *s ;
	int x, y, width ;
{
	printf("%d\n",width) ;
	printf("(%s) stringwidth\n",s) ;
	printf("pop sub 2 div %d add\n",x) ;
	printf("%d moveto\n", y) ;
	printf("(%s) show\n",s) ;
}

Caption()
{
	char string[100] ;

	readstring(string) ;
	setfont("Times-Roman",14) ;
	centerstring(string,Xmargin, 60, ImageSize) ;
	readstring(string) ;
	centerstring(string,Xmargin, 40, ImageSize) ;
}

dolabel()
{
	setfont("Helvetica-Bold",14) ;
	readstring(Label) ;
	centerstring(Label,Xmargin, Ymargin+LegendSpace-40, ImageSize) ;
}

StringAt(s, x, y)
	char *s ;
	int x, y ;
{
	printf("%d %d moveto\n",x, y) ;
	printf("(%s) show\n", s) ;
}

DoLegend(greyvalue, x, y, label)
{
	printf("%d %d translate\n",x, y) ;
	printf("%d %d scale\n",LegendDot, LegendDot) ;
	printf("1 1 8 [1 0 0 1 0 1]\n") ;
	printf("{currentfile dot readhexstring pop} image\n") ;
	printhex(greyvalue) ;
	putchar('\n') ;
	printf("grestore\n") ;
	setfont("Helvetica", 12) ;
	StringAt(label, x+LegendDot+15, y-LegendDot/2) ;
}

Legend()
{
	int x, y, greyvalue ;
	int label[100] ;

	printf("/dot 1 string def\n") ;
	x = Xmargin + 20 ;
	y = Ymargin + LegendSpace - 50 ;
	while (scanf("%d",&greyvalue) == 1) {
		readstring(label) ;
		DoLegend(greyvalue,x, y, label) ;
		y = y - LegendDot - 5 ;
		if (y < Ymargin) break ;
	}
	readln() ;
}

skipblanks()
{
	int c ;
	while ((c = getchar()) == ' ') ;
	ungetc(c,stdin) ;
}

getfield(s)
	char *s ;
{
	skipblanks() ;
	while ((*s = getchar()) != ' ' && *s != '\n') s++ ;
	*s = '\0' ;
}
/*
 *	int the form of horizontal keys
 */
#define keyheight 14
#define labelwidth 	50
#define labelgap	15
Legend2()
{
	char label[100], keystr[10];
	int nkeys, i, keylength, keyval ;

	readstring(label) ;
	setfont("Helvetica-Oblique",11) ;
	x = Xmargin + labelwidth ;
	y = Ymargin + LegendSpace - 50 ;
	LeftJustify("Key", x, y) ;
	LeftJustify(label, x, y - 15) ;

	keylength = ImageSize -  labelwidth -labelgap ;

	printf("/keystr 256 string def\n") ;
	printf("%d %d translate\n", x+labelgap, y-4) ;
	printf("%d %d scale\n", keylength, keyheight) ;
	printf("256 1 8 ") ;
	printf("[256 0 0 -1 0 1]\n") ;
	printf("{currentfile keystr readhexstring pop}\n") ;
	printf("image\n") ;
	for ( i = 0 ; i < 256; i++) {
		printhex(i) ;
	}
	putchar('\n') ;

	printf("\ngrestore\n") ;
	y = y - 15 ;
	x = x + labelgap ;
	setfont("Helvetica",9) ;
	scanf("%d",&nkeys) ;
	for ( i = 0; i <= nkeys; i++) {
		getfield(keystr) ;
		StringAt(keystr,x,y) ;
		x = x+(int)((float)keylength/nkeys+0.5) ;
	}
	readln() ;
}

main()
{
	int greyvalue ;

	scanf("%d %d",&height, &width) ;
	usersize = width > height ? width : height ;
	readln() ;
	readstring(A) ;
	readstring(B) ;
	PixelsPerDot = (float)ImageSize/usersize ;
	printf("initgraphics\n") ;
	printf("/picstr %d string def\n", width) ;
	printf("%d %d translate\n", Xmargin, (int)(Ymargin+LegendSpace-(usersize-height)*PixelsPerDot)) ;
	printf("%d %d scale\n", ImageSize, ImageSize) ;
	printf("%d %d 8 ",width, height) ;
	printf("[%d 0 0 %d 0 %d]\n",usersize,-usersize,usersize) ;
	printf("{currentfile picstr readhexstring pop}\n") ;
	printf("image\n") ;
	for ( i = 0 ; i < height; i++) {
		for ( j = 0; j < width; j++) {
			scanf("%d", &greyvalue) ;
			printhex(greyvalue) ;
		}
		putchar('\n') ;
		readln() ;
	}
	printf("\ngrestore\n") ;

	/*
	 *	Now print the strings alongside the density plot
	 */
	fontsize = FontSize(PixelsPerDot) ;
	setfont(FontType, fontsize) ;

	y = (int)(Ymargin + LegendSpace + PixelsPerDot*height+fontsize/2.0) ;
	for ( i = 0; i < width-1; i++) {
		x = (int)(Xmargin + (i+1)*PixelsPerDot-fontsize/4.0+0.5);
		printf("%d %d moveto",x,y) ;
		printf(" (%c) show\n",B[i]) ;
	}
	Ymax = (int)(Ymargin + LegendSpace + PixelsPerDot*height) ;

	x = (int)(Xmargin - fontsize);
	for ( i = 0 ; i < height-1; i++) {
		y = (int)(Ymargin + LegendSpace + PixelsPerDot*(height-i-1) - fontsize/3.0+0.5) ; 
		printf("%d %d moveto",x,y) ;
		printf(" (%c) show\n",A[i]) ;
	}
	Legend2() ;
	Caption() ;
	printf("showpage\n") ;
}
