
/*
 *  Copyright  (c)  L.Allison, C.S.Wallace, C.N.Yee.
 *     Dept. Computer Science, Monash University, Australia 3168,    May 1991.
 *  1. These routines may only be used for non-commercial, non-classified,
 *     research purposes.
 *  2. No liability of any kind is accepted for their use.
 *  3. As a courtesy, please inform us of any interesting uses of the routines,
 *     of any bugs that you find in them and of how to correct
 *     such bugs, if known.
 *  4. Do not delete this copyright notice.
 *  5. Please include the following references in any publication based wholly
 *     or in part on the use of these routines or on the ideas contained in them:
 *   . Allison L. & C.N.Yee.
 *        Minimum Message Length Encoding and the Comparison of Macro-Molecules.
 *        Bull. Math. Bio. 52(3) 431-453 1990
 *   . Allison L., C.S.Wallace & C.N.Yee. Finite-State Models in the Alignment
 *        of Macromolecules. J. Mol. Evol. (1992) 35:77-89
 *  6. You may distribute the routines to others if and only if
 *     they agree to accept and maintain these conditions and no fee is charged.
 *  7. There is no rule 7.
 */
/*
 *	"rtheory.h" definitions for the 1-3-5-State machine
 */
#define fwd 0 
#define rev 1 

#define adaptive	0 
#define fixed	 	1 

#define BIG	-1e7

/*
 *	definition for the 1State machine
 */
typedef struct {
		float MesLen ;                /* r-theory message length */
		float match, change, indel ; /* weighted frequency counters */
	} cell ;

typedef struct {
		float match, change, indel ;
	} OneStatePBlock ;

float OS_messagelength() ;

/*-------------------------------------------------------------------------
 *	definitions for 3 State machine
 */
typedef struct {
	float D, H, V ;		/* probs that the machine is in these states */
	float DM, DC, DH, DV ; /* probs of the arcs */
	float HM, HC, HH, HV ;
	float VM, VC, VH, VV ;
} ThreeStatePBlock ;

typedef struct {
	float D, H, V ;		/* cost of the three states */
	float DM, DC, DH, DV ; /*weighted frequency counters*/
	float HM, HC, HH, HV ;
	float VM, VC, VH, VV ;
} TScell ;

typedef struct {
	float D, H, V ;
} TSpcell ;

#define FREE 0
#define SYMM 1
/*
 * the free adapting version and the symmetric version (where H = V)
 */
#define ThreeStateML(A,B,MesgLength,MatrixPart,mode,outfile,P,Accurarcy) \
ThreeStateMesLength(A,B,MesgLength,MatrixPart,mode,outfile,P,Accurarcy,FREE)

#define ThreeStateMLS(A,B,MesgLength,MatrixPart,mode,outfile,P,Accurarcy)\
ThreeStateMesLength(A,B,MesgLength,MatrixPart,mode,outfile,P,Accurarcy,SYMM)


/*----------------------------------------------------------------------------
 *	declarations for 5-State machine with rabit ears long indel . "5State.c"
 */
typedef struct {
	float D, H, V, A, B ;		/* probs that the machine is in these states */
	float DM, DC, DH, DV, DA, DB ; /* probs of the arcs */
	float HM, HC, HH, HV ;
	float VM, VC, VH, VV ;
	float AD, AA ;
	float BD, BB ;
} FiveStatePBlock ;

typedef struct {
	float D, H, V, A, B ;		/* cost of the three states */
	float DM, DC, DH, DV, DA, DB ; /*weighted frequency counters*/
	float HM, HC, HH, HV ;
	float VM, VC, VH, VV ;
	float AD, AA ;
	float BD, BB ;
} FScell ;

typedef struct {
	float D, H, V, A, B ;
} FSpcell ;

#define FIXED	1	/* fix the probs of the long indels */
#define FiveStateML(A,B,ML,MatrixPart,mode,outfile,P,Accurarcy) \
	FiveStateMesgLength(A,B,ML,MatrixPart,mode,outfile,P,Accurarcy,FREE)
#define FiveState2ML(A,B,ML,MatrixPart,mode,outfile,P,Accurarcy) \
	FiveStateMesgLength(A,B,ML,MatrixPart,mode,outfile,P,Accurarcy,FIXED)
